@@ -24,7 +24,7 @@ class AlgorithmDaemon:
     __zmq_client = __zmq_context.socket(zmq.DEALER)
     # pylint: disable=maybe-no-member
     __zmq_client.setsockopt(zmq.IDENTITY, __client_identity.encode())
-    __is_stop = False
+    __is_safe_exit = False
 
     @staticmethod
     def get_instance():
@@ -53,20 +53,20 @@ class AlgorithmDaemon:
         logger.info("Daemon ready to shutdown.")
         self.__status = Status.Completed
         self.__send_status()
-        self.__is_stop = True
-        time.sleep(1)
+        self.wait_to_exit(10)
 
     def __on_client_recv(self):
-        while not self.__is_stop:
+        while not self.__is_safe_exit:
             msgs = self.__zmq_client.recv_multipart()
             if len(msgs) <= 0:
                 raise ValueError("Can not recv empty message.")
 
-            (action, *msgs) = msgs
-            self.__handle_action(action, msgs)
+            action, *msgs = msgs
+            self.__handle_action(action, *msgs)
 
     def __send(self, *args):
-        logger.info('Sending message: {}'.format('-'.join(args)))
+        logger.info('Sending message: {}'.format(
+            '-'.join([msg for msg in args if msg])))
         messages = []
         messages.extend([x.encode() for x in args])
         self.__zmq_client.send_multipart(messages)
@@ -82,10 +82,19 @@ class AlgorithmDaemon:
     def __handle_action(self, action, *args):
         if not action:
             raise ValueError("Action can not be empty.")
+        if action != b'ack':
+            raise ValueError(f"Action is not ack, [{action}] ")
 
-        if action == b'ack':
-            logger.info("Agent service acknowledged successful [ack].")
-            return
+        (ack_action, *args) = args
+        logger.info(
+            f"Agent service acknowledged successful [{action}] [{ack_action}].")
+
+        if ack_action == b'status':
+            status = args[0].decode('utf8')
+            logger.info(f"Agent service ack status [{status}]")
+            if status == Status.Completed or status == Status.Faulted:
+                logger.info(f"worker can exit safely.")
+                self.__is_safe_exit = True
 
     def report_progress(self, progress):
         if progress < 0 or progress > 1:
@@ -96,4 +105,13 @@ class AlgorithmDaemon:
         logger.error(exception)
         self.__status = Status.Faulted
         self.__send_status(str(exception))
-        time.sleep(1)
+        self.wait_to_exit(10)
+
+    def wait_to_exit(self, timeout):
+        start = time.time()
+        while not self.__is_safe_exit and time.time() - start < timeout:
+            time.sleep(0.1)
+        if self.__is_safe_exit:
+            logger.info("worker eixt.")
+        else:
+            logger.info("worker exit with timeout.")
